// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_PSEUDORANDOM_H
#define HU_VARADIISTVAN_SCPL_PSEUDORANDOM_H
//------------------------------------------------------------------------------

#include <cstdlib>
#include <ctime>

#include <inttypes.h>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl {

//------------------------------------------------------------------------------

/**
 * A pseudo-random number generator. The algorithm is the same as
 * described in http://www.mathstat.dal.ca/~selinger/random, i.e. the
 * GNU libc random number generator. It is reimplemented, because
 * Windows does not provide a thread-safe, reentrable pseudo-random
 * number generation facility, so it should have been implemented for
 * Win32 anyway.
 */
class PseudoRandom
{
public:
    /**
     * The largest random number generated in the unconverted
     * sequence.
     */
    static const uint32_t MAX = (1<<31);

private:
    /**
     * The size of the vector.
     */
    static const size_t vectorSize = 34;

    /**
     * The number of random numbers thrown away.
     */
    static const size_t numDiscarded = 344;

    /**
     * The vector of random numbers used to generate the next one.
     */
    uint32_t vector[vectorSize];

    /**
     * The index of the next random number.
     */
    int nextIndex;

public:
    /**
     * Construct the random number generator with a default seed
     * (taken by calling time()).
     */
    PseudoRandom();

    /**
     * Construct the random number generator with the given seed.
     */
    PseudoRandom(uint32_t seed);

    /**
     * Get the next raw value.
     */
    uint32_t next();

    /**
     * Get the next random number as an unsigned value between [from, to].
     */
    unsigned nextUnsigned(unsigned to = MAX, unsigned from=0);

    /**
     * Get the next random number as a double value between [from,
     * to].
     */
    double nextDouble(double to = 1.0, double from= 0.0);

private:
    /**
     * Initialize the random number generator with the given seed.
     */
    void initialize(uint32_t seed);
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline PseudoRandom::PseudoRandom()
{
    initialize(time(0));
}

//------------------------------------------------------------------------------

inline PseudoRandom::PseudoRandom(uint32_t seed)
{
    initialize(seed);
}

//------------------------------------------------------------------------------

inline uint32_t PseudoRandom::next()
{
    int i1 = (nextIndex<3) ? (vectorSize + nextIndex - 3) : (nextIndex - 3);
    int i2 = (nextIndex<31) ? (vectorSize + nextIndex - 31) : (nextIndex - 31);

    vector[nextIndex] = vector[i1] + vector[i2];
    uint32_t x = vector[nextIndex]>>1;

    ++nextIndex; if (nextIndex>=static_cast<int>(vectorSize)) nextIndex = 0;

    return x;
}

//------------------------------------------------------------------------------

inline unsigned PseudoRandom::nextUnsigned(unsigned to, unsigned from)
{
    return from + static_cast<unsigned>(static_cast<double>(to - from) * next()
                                        / static_cast<double>(MAX));
}

//------------------------------------------------------------------------------

inline double PseudoRandom::nextDouble(double to, double from)
{
    return from + (to-from) * next() / static_cast<double>(MAX);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_PSEUDORANDOM_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
